/*
 * 			Combinations
 * 
 * 			Combinations.cpp
 * 
 * 							2005 by Harald Obermaier
 */


#ifdef WIN32
#define WIN32_LEAN_AND_MEAN
#include <windows.h>
#endif

#include <SDL.h>
#include <stdlib.h>

//Shouldn't change resolution, since I'm not using relative coordinates for positioning
#define WIDTH 	800
#define HEIGHT	600


#ifdef WIN32
	#include "..\headers\Combinations.h"
#else
	#include "../headers/Combinations.h"
#endif

extern SDL_Surface *cones;
extern SDL_Surface *gui;
extern SDL_Surface *buffer;
extern bool done;

//Macro to check whether ix,iy is within the borders of rect
#define ON(ix,iy,rect) (ix>=rect.x && ix <= rect.x+rect.w && iy>= rect.y && iy <= rect.y+rect.h)


void GuiElement::Draw()
{
	SDL_BlitSurface(gui,&bmp,buffer,&screen);	
}

bool GuiElement::OnButton(int x, int y)
{
	//returns, whether this is a button and x,y is "on" the button's outline-rect
	return (button && ON(x,y,outline));
}

void GuiElement::setPos(SDL_Rect bmp1, SDL_Rect screen1, SDL_Rect outline1)
{
	bmp = bmp1;
	screen = screen1;	
	outline = outline1;
}

void GuiElement::setButton(bool b)
{
	button = b;	
}

void GuiElement::DrawOutline()
{
	//blue rectangle
	SDL_FillRect(buffer, &outline, SDL_MapRGB(cones->format, 0, 0, 255));

	//black rectangle
	SDL_Rect screen = outline;
	screen.x++;
	screen.w-=2;
	screen.y++;
	screen.h-=2;
	SDL_FillRect(buffer, &screen, SDL_MapRGB(cones->format, 0, 0, 0));
	
}









void Guess::Reset()
{
	for(int i = 0;i < 4;i++)
	{
		combination[i] = -1;
		rating[i] = 0;
	}
}


int Guess::Rate(int c[4])
{
	int red = 0, white = 0;	//amount of red / white "ratings"
	
	for(int i = 0;i < 4;i++)
		if(combination[i] == -1) //invalid combination -> do not rate it
			return -1;
			
	for(int i = 0;i < 4;i++)
		for(int u = 0;u < 4;u++)
			if(c[i] == combination[u])
			{
				if(i == u)	//this color is at the right position
					red++;
				else		//this color is part of the goal-combination - wrong position
					white++;
				break;
			}
			
	for(int i = 0;i < white;i++)
		rating[i] = WHITE;
	for(int i = white; i < white+red;i++)
		rating[i] = RED;
	for(int i = white+red;i < 4;i++)
		rating[i] = 0;
		
	if(red == 4) //guessed right combination
		return 1;
		
	return 0;
}


int Guess::getColor(int i)
{
	return combination[i];	
}


void Guess::setColor(int i,int color)
{
	combination[i] = color;	
}


int Guess::getRating(int i)
{
	return rating[i];	
}









Combinations::Combinations()
{
	if(SDL_Init(SDL_INIT_VIDEO) == -1) //Initialize SDL
	{
		fprintf(stderr, "Initialization of SDL failed: %s\n", SDL_GetError());
		done = true;
		return;
	}
	
	
	if(!(buffer = SDL_SetVideoMode(WIDTH,HEIGHT,16,SDL_SWSURFACE)))//Set video mode to WIDTHxHEIGHTx16
	{
		fprintf(stderr, "Setting video-mode failed: %s\n", SDL_GetError());
		done = true;
		return;
	}
	SDL_WM_SetCaption("Combinations",NULL);		//Set window title to "Combinations"
	
	mousex = mousey = win =  0;
	selcolor = -1;
	
	//Initialize Buttons/Labels - coordinates are absolute
	//Rate
	{
		SDL_Rect dstrect = {687,540,55,25};
		SDL_Rect srcrect = {0,0,53,22};
		SDL_Rect outline = {645,535,135,30};
		buttons[0].setPos(srcrect,dstrect,outline);
		buttons[0].setButton(true);
	}
	//New Game
	{
		SDL_Rect dstrect = {652,20,125,20};
		SDL_Rect srcrect = {0,25,125,20};
		SDL_Rect outline = {645,15,135,30};
		buttons[1].setPos(srcrect,dstrect,outline);
		buttons[1].setButton(true);
	}
	//Quit
	{
		SDL_Rect dstrect = {687,60,50,25};
		SDL_Rect srcrect = {0,72,50,22};
		SDL_Rect outline = {645,55,135,30};
		buttons[2].setPos(srcrect,dstrect,outline);
		buttons[2].setButton(true);
	}
	//You Win
	{
		SDL_Rect dstrect = {350,200,100,23};
		SDL_Rect srcrect = {129,28,100,22};
		SDL_Rect outline = {350,200,100,23};
		buttons[3].setPos(srcrect,dstrect,outline);
		buttons[3].setButton(false);
	}
	//Start a new game ?
	{
		SDL_Rect dstrect = {300,250,215,23};
		SDL_Rect srcrect = {0,98,215,26};
		SDL_Rect outline = {300,250,215,23};
		buttons[4].setPos(srcrect,dstrect,outline);
		buttons[4].setButton(false);
	}
	//Yes
	{
		SDL_Rect dstrect = {305,300,46,23};
		SDL_Rect srcrect = {0,50,46,23};
		SDL_Rect outline = {300,295,55,33};
		buttons[5].setPos(srcrect,dstrect,outline);
		buttons[5].setButton(true);
	}
	//No
	{
		
		SDL_Rect dstrect = {470,300,33,23};
		SDL_Rect srcrect = {48,50,33,23};
		SDL_Rect outline = {465,295,42,33};
		buttons[6].setPos(srcrect,dstrect,outline);
		buttons[6].setButton(true);
	}
	//You Lose
	{
		
		SDL_Rect dstrect = {350,200,100,23};
		SDL_Rect srcrect = {128,0,100,23};
		SDL_Rect outline = {350,200,100,23};
		buttons[7].setPos(srcrect,dstrect,outline);
		buttons[7].setButton(false);
	}
}

Combinations::~Combinations()
{
}



void Combinations::NewGame()
{
	
	SDL_WM_SetCaption("Combinations",NULL);
	for(int i = 0;i < 10;i++)
		guesses[i].Reset();		//Reset guesses
	
	srand(SDL_GetTicks());		//Generate a set of new random numbers
	for(int i = 0;i < 4;i++)
	{
		combination[i] = rand()%9;	//Random color (0-8)
		for(int j = 0;j < i;j++)
		{
			if(combination[i] == combination[j])//no double-colors - generate color again
			{
				i--;
				break;	
			}
		}
	}
	
	win = turn = 0;
	selcolor = -1;				//Not dragging a color
	starttime = SDL_GetTicks();	//Save time of start
		
}




void Combinations::Render()
{
	
	SDL_Rect screen = {0,0,WIDTH,HEIGHT};
	SDL_FillRect(buffer, &screen, SDL_MapRGB(cones->format, 0, 0, 0));
	
	
	
	SDL_Rect srcrect = {0,0,32,32};
	SDL_Rect dstrect = {0,0,32,32};
	
	//Render board: cones / holes
	for(int i = 0;i < 10;i++)
	{
		dstrect.y = 550 - 60*i;
		for(int j = 0; j < 4;j++)
		{
			dstrect.x = 320+45*j;
			srcrect.x = guesses[i].getColor(j)*32+32;
			SDL_BlitSurface(cones,&srcrect,buffer,&dstrect);
										
		}
	}
	
	
	dstrect.x = 520;	
	dstrect.y = 550 - 60*turn;

	srcrect.x = 392;	
	srcrect.y = 0;
	SDL_BlitSurface(cones,&srcrect,buffer,&dstrect); //Randomize - Button
	
	if(ON(mousex,mousey,dstrect))//Outline of Randomize-button
	{
		srcrect.x = 360;
		SDL_BlitSurface(cones,&srcrect,buffer,&dstrect);
	}
	
	
	if(win == -1)//we lost - display combination
	{
		dstrect.y = 290;
		srcrect.y = 0;
		for(int j = 0; j<4;j++)
		{
			dstrect.x = 600+45*j;
			srcrect.x = combination[j]*32+32;
			SDL_BlitSurface(cones,&srcrect,buffer,&dstrect); //cones
											
		}
	}
	
	//colors-list to the left
	dstrect.x = 100;
	srcrect.y = 0;
	for(int i = 0;i<9;i++)
	{
		dstrect.y = 45+i*60;
		srcrect.x = i*32+32;
		SDL_BlitSurface(cones,&srcrect,buffer,&dstrect); 
	}
	if(mousex > 100 && mousex < 135 && mousey > 45 && mousey <585) //highlight color for Drag&Drog
	{
		dstrect.y = 45+((mousey-45)/60)*60;
		dstrect.x = 100;
			
		srcrect.y = 0;
		srcrect.x = 360;
		SDL_BlitSurface(cones,&srcrect,buffer,&dstrect); 
		
	}
	else if(mousex > 320 && mousex < 487 && mousey > 550-60*turn && mousey < 582-60*turn) //highlight position for Drag&Drog
	{
		dstrect.y = 550-60*turn;
		dstrect.x = 319+((mousex-320)/45)*45;
			
		srcrect.y = 0;
		srcrect.x = 360;
		SDL_BlitSurface(cones,&srcrect,buffer,&dstrect);
	}
	
	
	//Display ratings
	srcrect.h = srcrect.w = dstrect.h = dstrect.w =10;
	srcrect.y = 0;
	for(int i = 0;i < 10;i++)
	{
		for(int j = 0; j < 4;j++)
		{
			dstrect.y = 600-50 - 60*i + 15*(j>1?1:0)+5;
			dstrect.x = 250+10*(j%2);
			
			srcrect.x = 324+((i>=turn) ? 0 : guesses[i].getRating(j))*10;
			SDL_BlitSurface(cones,&srcrect,buffer,&dstrect); //ratings
										
		}
	}
	
	//Draw "New Game", "Quit" and "Rate"	
	for(int i = 0;i < 3;i++) 
	{
		if(buttons[i].OnButton(mousex,mousey))
			buttons[i].DrawOutline();
		buttons[i].Draw();
	}
	
	
	
		
	if(selcolor != -1)	//we are dragging a color
	{
		dstrect.x =	mousex;
		dstrect.y = mousey;
		
		srcrect.h = dstrect.h = srcrect.w = dstrect.w = 32;
		
		srcrect.x = selcolor*32+32;	
		srcrect.y = 0;
		
		SDL_BlitSurface(cones,&srcrect,buffer,&dstrect);
	}
	
	if(win)
	{
		if(win > 0)
			buttons[3].Draw();//Displays "You Win"
		if(win < 0)
			buttons[7].Draw();//Displays "You Lose"
			
		for(int i = 4;i<7;i++)
		{
			if(buttons[i].OnButton(mousex,mousey))
				buttons[i].DrawOutline();
			buttons[i].Draw();
		}
		
	}
	SDL_Flip(buffer);//Swap buffers
	
	
}


void Combinations::Rate()
{
	int rate = guesses[turn].Rate(combination);
	if(!rate)
		turn++;	//Combination is not the right one and we have still at least one turn left
		
	if(rate == 1)//Combination is right
	{
		starttime = SDL_GetTicks()-starttime;
		//Change title of window to: "Your time: [time]"
		char temp[64];
		sprintf(temp,"Your time:  %d : %d : %d",((starttime/1000)/60),(starttime/1000)%60,(starttime%1000)/10);
		SDL_WM_SetCaption(temp,NULL);
		win = 1;
		selcolor = -1;
		turn++;
	}
	else if(turn == 10) //we didn't get the right combination within 10 turns
	{
		win = -1;
		selcolor = -1;
	}
	
}



void Combinations::Randomize()
{
	for(int i = 0;i < 4;i++)
	{
		int r = rand()%9;	//Generate random color (0-8)
		int j = 0;
		for(;j < 4;j++)
		{
			if(guesses[turn].getColor(j) == r)//prevent combination from having a color twice
			{
				i--;
				break;
			}
		}
		if(j == 4)
			guesses[turn].setColor(i,r);
	}
}

void Combinations::MouseDown(SDL_MouseButtonEvent click)
{
	if(!win)
	{
		if(click.x >= 100 && click.x <= 135 && click.y >= 45 && click.y <= 585)
			selcolor = (click.y-45)/60; //select a color
		else if(click.x >= 320 && click.x <= 535 && click.y >= 550-turn*60 && click.y <= 585-turn*60)
		{
			//place a color
			for(int i = 0;i < 4;i++)
			{
				if(guesses[turn].getColor(i) == selcolor)
					guesses[turn].setColor(i,-1);
			}
			guesses[turn].setColor((click.x-320)/45,selcolor);
			selcolor = -1; //Cancel Drag&Drop
		}
		else
			selcolor = -1;	//Cancel Drag&Drop
	}
	
		
}

void Combinations::MouseUp(SDL_MouseButtonEvent click)
{
	if(win)
	{
		if(buttons[5].OnButton(mousex,mousey))
			NewGame();
		if(buttons[6].OnButton(mousex,mousey))
			done = true;
	}
	else
	{
		if(buttons[0].OnButton(mousex,mousey))
			Rate();
		if(buttons[1].OnButton(mousex,mousey))
			NewGame();
		if(buttons[2].OnButton(mousex,mousey))
			done = true;
		if(mousex > 520 && mousex < 552 && mousey > 550 - 60*turn && mousey < 550 - 60*turn+32)
			Randomize();
	
	}
	
	
}

void Combinations::MouseMove(SDL_MouseMotionEvent move)
{
	mousex = move.x;
	mousey = move.y;
}

int Combinations::Input()
{
	SDL_Event event;
	while(SDL_PollEvent(&event))
	{
		switch(event.type)
		{
			case SDL_KEYDOWN:
				
				if(event.key.keysym.sym == SDLK_ESCAPE)
					return -1;
				else if(event.key.keysym.sym == SDLK_n)
					NewGame();
				else if(!win)
				{
					if(event.key.keysym.sym == SDLK_r)
						Randomize();
					else if(event.key.keysym.sym == SDLK_RETURN)
						Rate();
				}
						
			break;
			case SDL_MOUSEBUTTONDOWN:
				MouseDown(event.button);
			break;
			case SDL_MOUSEBUTTONUP:
				MouseUp(event.button);
			break;
			case SDL_MOUSEMOTION:
				MouseMove(event.motion);
			break;
		}
	}
	return 0;	
}
